#ifndef _IDDEEVT_
#define _IDDEEVT_
/*******************************************************************************
* FILE NAME: iddeevt.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IDDEEvent                   - Base DDE event class.                      *
*     IDDEAcknowledgeEvent        - Base DDE acknowledgement class.            *
*     IDDESetAcknowledgeInfoEvent - Base class for all events that need to set *
*                                   acknowledgement information.               *
*     IDDEClientAcknowledgeEvent  - Information for acknowledgements from a    *
*                                   server to requestData(), beginHotLink()    *
*                                   and endHotLink() requests.                 *
*     IDDEServerAcknowledgeEvent  - Information for acknowledgements from a    *
*                                   client.                                    *
*     IDDEAcknowledgePokeEvent    - Information for acknowledgements from a    *
*                                   server to pokeData() requests.             *
*     IDDEAcknowledgeExecuteEvent - Information for acknowledgements from a    *
*                                   server to executeCommands() requests.      *
*     IDDEDataEvent               - Information and data from a server in      *
*                                   response to requestData() requests and as  *
*                                   a result of beginHotLink() requests.       *
*     IDDEClientHotLinkEvent      - Information from a server as a result of   *
*                                   beginHotLink() requests.                   *
*     IDDERequestDataEvent        - Information for requestData() requests     *
*                                   from a client.                             *
*     IDDEPokeEvent               - Information for pokeData() requests from   *
*                                   a client.                                  *
*     IDDEServerHotLinkEvent      - Information for beginHotLink() requests    *
*                                   from a client.                             *
*     IDDEExecuteEvent            - Information for executeCommands() requests *
*                                   from a client.                             *
*     IDDEEndEvent                - Base DDE end conversation event class.     *
*     IDDEClientEndEvent          - Information for a client about the ending  *
*                                   of a conversation.                         *
*     IDDEBeginEvent              - Information for a server about             *
*                                   beginConversation() requests.              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:
class IDDEClientConversation;
class IDDETopicServer;

class IDDEEvent : protected IEvent {
typedef IEvent
  Inherited;
/*******************************************************************************
* IDDEEvent is the base class for most dynamic data exchange (DDE) event       *
* classes.                                                                     *
*                                                                              *
* An instance of this class, or one of its subclasses, is created when an      *
* instance of IDDEClientConversation or IDDETopicServer needs to pass          *
* information to the client or server application.  Instances of this class    *
* are not intended to be created by application programmers.                   *
*******************************************************************************/
public:
/*---------------------- Constructor -------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEEvent  ( const IEvent&    ddeEvent );
  IDDEEvent  ( const IDDEEvent& ddeEvent );

  ~IDDEEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get the accessible attributes of instances    |
| of this class:                                                               |
|   item   - Returns the name of the item.                                     |
|   format - Returns the name of the format.                                   |
------------------------------------------------------------------------------*/
IString
  item   ( ) const,
  format ( ) const;

protected:
/*------------------------------ Implementation --------------------------------
| The following functions allow a subclass to get and set the accessible       |
| attributes of instances of this class:                                       |
|   setStatus - Sets the status field of the event.                            |
|   setBuffer - Sets the data into the data buffer.                            |
|   status    - Returns the status of the event.                               |
|   buffer    - Returns the data buffer.                                       |
|               NOTE:  The buffer function's IString is constructed from a     |
|                      void* and length.  For character strings, if the        |
|                      terminating NULL was included in the length, it will    |
|                      be in the buffer.  Before performing string             |
|                      operations such as concatenation on this string,        |
|                      strip any trailing NULL. (See IString::stripTrailing.)  |
------------------------------------------------------------------------------*/
IDDEEvent
 &setStatus ( unsigned short status ),
 &setBuffer ( IString        dataBuffer );

unsigned short
  status ( ) const;

IString
  buffer ( ) const;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
friend class IDDEClientConversation;
friend class IDDETopicServer;

IDDEEvent
 &setFormat ( IString format );

unsigned short
  usClStatus;
IString
  strClItem,
  strClFormat,
  strClData;

}; // IDDEEvent

class IDDEAcknowledgeEvent : public IDDEEvent {
typedef IDDEEvent
  Inherited;
/*******************************************************************************
* The IDDEAcknowledgeEvent class is the base class for the dynamic data        *
* exchange (DDE) acknowledge event information classes.                        *
*                                                                              *
* An instance of this class or one of its subclasses is created when an        *
* instance of IDDEClientConversation or IDDETopicServer needs to pass          *
* acknowledgement information to the client or server application.             *
* Instances of this class are not intended to be created by application        *
* programmers.                                                                 *
*******************************************************************************/
public:
/*------------------- Constructor ----------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEAcknowledgeEvent  ( const IEvent& ddeEvent );

  ~IDDEAcknowledgeEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get the accessible attributes of instances    |
| of this class:                                                               |
|   isApplicationBusy       - Returns true if the application busy flag is on; |
|                             otherwise, it returns false. This will only be   |
|                             set for a negative acknowledgement.              |
|   isAckPositive           - Returns true if the acknowledgement is positive; |
|                             otherwise, it returns false.                     |
|   isMessageUnderstood     - Returns false if the message not understood flag |
|                             is on; otherwise, it returns true.  This is      |
|                             only set for a negative acknowledgement.         |
|   applicationSpecificData - Returns any application-specific data that has   |
|                             been provided by the conversing application.     |
------------------------------------------------------------------------------*/
Boolean
  isApplicationBusy   ( ) const,
  isAckPositive       ( ) const,
  isMessageUnderstood ( ) const;

unsigned char
  applicationSpecificData ( ) const;

protected:
/*------------------------------ Implementation --------------------------------
| The following function allows a subclass to get the accessible attribute     |
| of instances of this class:                                                  |
|   transactionType - Returns the event ID.                                    |
------------------------------------------------------------------------------*/
unsigned short
  transactionType ( ) const;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
friend class IDDEClientConversation;
friend class IDDETopicServer;

}; // IDDEAcknowledgeEvent

class IDDESetAcknowledgeInfoEvent : public IDDEEvent {
typedef IDDEEvent
  Inherited;
/*******************************************************************************
* The IDDESetAcknowledgeInfoEvent class is the base class for dynamic data     *
* exchange (DDE) event information classes that need to set acknowledgement    *
* information.                                                                 *
*                                                                              *
* An instance of a subclass of IDDESetAcknowledgeInfoEvent is only created     *
* for events that the application can respond to with a negative               *
* acknowledgement.  Instances of this class are not typically created and are  *
* not intended to be created by application programmers.                       *
*******************************************************************************/
public:
/*------------------- Constructor ----------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDESetAcknowledgeInfoEvent  ( const IEvent&                      ddeEvent );
  IDDESetAcknowledgeInfoEvent  ( const IDDESetAcknowledgeInfoEvent& ddeEvent );

  ~IDDESetAcknowledgeInfoEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to set the accessible attributes of instances    |
| of this class:                                                               |
|   setApplicationBusy         - Sets the application busy flag.  This is      |
|                                used by a client or server application to     |
|                                indicate why a request or response to a       |
|                                request cannot be processed.                  |
|   setMessageNotUnderstood    - Sets the message not understood flag.  This   |
|                                is used by a client or server application to  |
|                                indicate why a request or response to a       |
|                                request cannot be processed.                  |
|   setApplicationSpecificData - Sets application-specific information.  This  |
|                                can be used when a client and server          |
|                                application have a pre-defined protocol for   |
|                                exchanging information.                       |
------------------------------------------------------------------------------*/
IDDESetAcknowledgeInfoEvent
 &setApplicationBusy         ( ),
 &setMessageNotUnderstood    ( ),
 &setApplicationSpecificData ( unsigned char applicationData );

}; // IDDESetAcknowledgeInfoEvent

class IDDEClientAcknowledgeEvent : public IDDEAcknowledgeEvent {
typedef IDDEAcknowledgeEvent
  Inherited;
/*******************************************************************************
* The IDDEClientAcknowledgeEvent class provides event information to a         *
* client application regarding an acknowledgement from a server.               *
*                                                                              *
* An instance of this class or one of its subclasses is created when an        *
* instance of IDDEClientConversation needs to pass information about an        *
* acknowledgement to the client application.  Instances of this class are not  *
* intended to be created by application programmers.                           *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEClientAcknowledgeEvent  ( const IEvent& ddeEvent );

  ~IDDEClientAcknowledgeEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get the accessible attributes of instances    |
| of this class:                                                               |
|   isAckToRequestData  - Returns true if the acknowledgement is in response   |
|                         to requestData being invoked; otherwise, it returns  |
|                         false.                                               |
|   isAckToBeginHotLink - Returns true if the acknowledgement is in response   |
|                         to beginHotLink being invoked; otherwise, it         |
|                         returns false.                                       |
|   isAckToEndHotLink   - Returns true if the acknowledgement is in response   |
|                         to endHotLink being invoked; otherwise, it returns   |
|                         false.                                               |
------------------------------------------------------------------------------*/
Boolean
  isAckToRequestData  ( ) const,
  isAckToBeginHotLink ( ) const,
  isAckToEndHotLink   ( ) const;

}; // IDDEClientAcknowledgeEvent

class IDDEServerAcknowledgeEvent : public IDDEAcknowledgeEvent {
typedef IDDEAcknowledgeEvent
  Inherited;
/*******************************************************************************
* The IDDEServerAcknowledgeEvent class provides event information to a         *
* server application regarding an acknowledgement from a client.               *
*                                                                              *
* An instance of this class is created when an instance of IDDETopicServer     *
* needs to pass information about an acknowledgement to the server             *
* application.  Instances of this class are not intended to be created by      *
* application programmers.                                                     *
*******************************************************************************/
public:
/*---------------------- Constructor -------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEServerAcknowledgeEvent  ( const IEvent& ddeEvent );

  ~IDDEServerAcknowledgeEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get the accessible attributes                 |
| of instances of this class:                                                  |
|   isAckToHotLinkUpdate - Returns true if the acknowledgement is in response  |
|                          to a hotLinkUpdate being invoked.  It returns       |
|                          false if it is in response to data being provided   |
|                          for a requestData request.                          |
|   data                 - Returns the data the server provided to the client. |
|                          NOTE:  The data function's IString is constructed   |
|                                 from a void* and length.  For character      |
|                                 strings, if the terminating NULL was         |
|                                 included in the length, it will be in the    |
|                                 buffer.  Before performing string            |
|                                 operations such as concatenation on this     |
|                                 string, strip any trailing NULL.             |
|                                 (See IString::stripTrailing.)                |
------------------------------------------------------------------------------*/
Boolean
  isAckToHotLinkUpdate ( ) const;

IString
  data ( ) const;

}; // IDDEServerAcknowledgeEvent

class IDDEAcknowledgePokeEvent : public IDDEClientAcknowledgeEvent {
typedef IDDEClientAcknowledgeEvent
  Inherited;
/*******************************************************************************
* The IDDEAcknowledgePokeEvent class provides event information to a client    *
* application regarding an acknowledgement to a pokeData request.              *
*                                                                              *
* An instance of this class is created when an instance of                     *
* IDDEClientConversation needs to pass information to the client application   *
* about an acknowledgement resulting from a pokeData request.  Instances of    *
* this class are not intended to be created by application programmers.        *
*******************************************************************************/
public:
/*-------------------- Constructor --------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEAcknowledgePokeEvent  ( const IEvent& ddeEvent );

  ~IDDEAcknowledgePokeEvent ( );

/*-------------------------------- Accessor ------------------------------------
| Use the following function to get the accessible attribute of instances of   |
| this class:                                                                  |
|   pokedData - Returns the data that the client asked the server application  |
|               to set.                                                        |
|               NOTE:  The pokedData function's IString is constructed from a  |
|                      void* and length.  For character strings, if the        |
|                      terminating NULL was included in the length it will     |
|                      be in the buffer.  Before performing string             |
|                      operations such as concatenation on this string,        |
|                      strip any trailing NULL.  (See IString::stripTrailing.) |
-------------------------------------------------------------------------------*/
IString
  pokedData ( ) const;

}; // IDDEAcknowledgePokeEvent

class IDDEAcknowledgeExecuteEvent : public IDDEClientAcknowledgeEvent {
typedef IDDEClientAcknowledgeEvent
  Inherited;
/*******************************************************************************
* The IDDEAcknowledgeExecuteEvent class provides event information to a        *
* client application regarding an acknowledgement to an executeCommands        *
* request.                                                                     *
*                                                                              *
* An instance of this class is created when an instance of                     *
* IDDEClientConversation needs to pass information to the client application   *
* about an acknowledgement resulting from an executeCommands request.          *
* Instances of this class are not intended to be created by application        *
* programmers.                                                                 *
*******************************************************************************/
public:
/*--------------------- Constructor --------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEAcknowledgeExecuteEvent  ( const IEvent& ddeEvent );

  ~IDDEAcknowledgeExecuteEvent ( );

/*-------------------------------- Accessor ------------------------------------
| Use the following function to get the accessible attribute of instances of   |
| this class:                                                                  |
|   commands - Returns the command string that the client asked the server     |
|              application to execute.                                         |
|              NOTE:  The commands function's IString is constructed from a    |
|                     void* and length.  For character strings, if the         |
|                     terminating NULL was included in the length it will be   |
|                     in the buffer.  Before performing string operations      |
|                     such as concatenation on this string, strip any          |
|                     trailing NULL.  (See IString::stripTrailing.)            |
------------------------------------------------------------------------------*/
IString
  commands ( ) const;

}; // IDDEAcknowledgeExecuteEvent

class IDDEDataEvent : public IDDESetAcknowledgeInfoEvent {
typedef IDDESetAcknowledgeInfoEvent
  Inherited;
/*******************************************************************************
* The IDDEDataEvent class provides event information and data to a client      *
* application.                                                                 *
*                                                                              *
* An instance of this class is created when an instance of                     *
* IDDEClientConversation needs to pass data sent from a server to the client   *
* application.  This can result from a requestData request or from an active   *
* hot link.  Instances of this class are not intended to be created by         *
* application programmers.                                                     *
*******************************************************************************/
public:
/*-------------------- Constructor ---------------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEDataEvent  ( const IEvent& ddeEvent );

  ~IDDEDataEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get accessible attributes of instances of     |
| this class:                                                                  |
|   data              - Returns the data provided by the server.               |
|                       NOTE:  The data function's IString is constructed      |
|                              from a void* and length.  For character         |
|                              strings, if the terminating NULL was included   |
|                              in the length, it will be in the buffer.        |
|                              Before performing string operations such as     |
|                              concatenation on this string, strip any         |
|                              trailing NULL.  (See IString::stripTrailing.)   |
|   isAckRequested    - Returns true if the server has requested an            |
|                       acknowledgement of receipt of the data.  The           |
|                       IDDEClientConversation instance will automatically     |
|                       send the acknowledgement when it is requested.         |
|   isDataFromHotLink - Returns true if the data is being sent as the result   |
|                       of an active hot link. Returns false if the data is    |
|                       sent in response to a requestData request.             |
------------------------------------------------------------------------------*/
IString
  data ( ) const;

Boolean
  isAckRequested    ( ) const,
  isDataFromHotLink ( ) const;

}; // IDDEDataEvent

class IDDEClientHotLinkEvent : public IDDESetAcknowledgeInfoEvent {
typedef IDDESetAcknowledgeInfoEvent
  Inherited;
/*******************************************************************************
* The IDDEClientHotLinkEvent class provides information to a client            *
* application when a hot link item change notification is sent by a server     *
* application.                                                                 *
*                                                                              *
* An instance of this class is created when an instance of                     *
* IDDEClientConversation needs to pass a hot line notification to the client   *
* application. An instance of this class for each active hot link is also      *
* kept in a set by IDDEClientConversation instances to keep track of active    *
* hot links.  Instances of this class are not intended to be created by        *
* application programmers.                                                     *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEClientHotLinkEvent  ( const IEvent&                 ddeEvent );
  IDDEClientHotLinkEvent  ( const IDDEClientHotLinkEvent& ddeEvent );

  ~IDDEClientHotLinkEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get the the accessible attributes             |
| of instances of this class:                                                  |
|   isAckRequested  - Returns true if the server has requested an              |
|                     acknowledgement of receipt of the notification.          |
|                     The IDDEClientConversation instance will automatically   |
|                     send the acknowledgement when it is requested.           |
|   isDataRequested - Returns true if the hot link is a data hot link.         |
|                     False is returned if it is a notification hot link.      |
|                     This function is only pertinent for instances of this    |
|                     class used to keep track of active hot links (contained  |
|                     in an IDDEClientHotLinkSet instance).  All other         |
|                     instances of this class will be created for              |
|                     notifications sent by non-data hot links.                |
------------------------------------------------------------------------------*/
Boolean
  isAckRequested  ( ) const,
  isDataRequested ( ) const;

}; // IDDEClientHotLinkEvent

class IDDERequestDataEvent : public IDDESetAcknowledgeInfoEvent {
typedef IDDESetAcknowledgeInfoEvent
  Inherited;
/*******************************************************************************
* The IDDERequestDataEvent class provides event information to a server        *
* application when a client has requested data.                                *
*                                                                              *
* An instance of this class is created when an instance of IDDETopicServer     *
* needs to pass requests for data to the server application.  Instances of     *
* this class are not intended to be created by application programmers.        *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDERequestDataEvent  ( const IEvent& ddeEvent );

  ~IDDERequestDataEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get and set the accessible attributes of      |
| instances of this class:                                                     |
|   requestAck - Sets the request acknowledgement flag.                        |
|   setData    - Provides the requested data so it can be sent to the client.  |
|                This function is overloaded so that either buffers of data    |
|                or character strings can be easily sent.                      |
------------------------------------------------------------------------------*/
IDDERequestDataEvent
 &requestAck ( ),
 &setData    ( const char*   dataString ),
 &setData    ( const void*   dataBuffer,
               unsigned long dataLength );

}; // IDDERequestDataEvent

class IDDEPokeEvent : public IDDESetAcknowledgeInfoEvent {
typedef IDDESetAcknowledgeInfoEvent
  Inherited;
/*******************************************************************************
* The IDDEPokeEvent class provides event information to a server application
* when a client has asked the server to set an item to a new value.                          *
*                                                                              *
* An instance of this class is created when an instance of IDDETopicServer     *
* needs to pass a pokeData request to the server application.  Instances of    *
* this class are not intended to be created by application programmers.        *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEPokeEvent  ( const IEvent& ddeEvent );

  ~IDDEPokeEvent ( );

/*-------------------------------- Accessor ------------------------------------
| Use the following function to get the accessible attribute of instances of   |
| this class:                                                                  |
|   pokedData - Returns the data that the client asked the server to set       |
|               an item's value to.                                            |
|               NOTE:  The pokedData function's IString is constructed from    |
|                      a void* and length.  For character strings, if the      |
|                      terminating NULL was included in the length, it will    |
|                      be in the buffer.  Before performing string             |
|                      operations such as concatenation on this string,        |
|                      strip any trailing NULL.  (See IString::stripTrailing.) |
------------------------------------------------------------------------------*/
IString
  pokedData ( ) const;

}; // IDDEPokeEvent

class IDDEExecuteEvent : public IDDESetAcknowledgeInfoEvent {
typedef IDDESetAcknowledgeInfoEvent
  Inherited;
/*******************************************************************************
* The IDDEExecuteEvent class provides event information to a server            *
* application when a client has asked the server to execute a command string.  *
*                                                                              *
* An instance of this class is created when an instance of IDDETopicServer     *
* needs to pass an executeCommands request to the server application.          *
* Instances of this class are not intended to be created by application        *
* programmers.                                                                 *
*******************************************************************************/
public:
/*------------------------- Constructor ---------------------------------------
| There is one way to create instances of this class:                          |
------------------------------------------------------------------------------*/
  IDDEExecuteEvent  ( const IEvent& ddeEvent );

  ~IDDEExecuteEvent ( );

/*-------------------------------- Accessor ------------------------------------
| Use the following function to get the accessible attribute of instances of   |
| this class:                                                                  |
|   commands - Returns the command string that the client has asked the        |
|              server to execute.                                              |
|              NOTE:  The commands function's IString is constructed from a    |
|                     void* and length.  For character strings, if the         |
|                     terminating NULL was included in the length, it will     |
|                     be in the buffer.  Before performing string operations   |
|                     such as concatenation on this string, strip any          |
|                     trailing NULL. (See IString::stripTrailing.)             |
------------------------------------------------------------------------------*/
IString
  commands ( ) const;

}; // IDDEExecuteEvent

class IDDEServerHotLinkEvent : public IDDESetAcknowledgeInfoEvent {
typedef IDDESetAcknowledgeInfoEvent
  Inherited;
/*******************************************************************************
* The IDDEServerHotLinkEvent class provides event information to a server      *
* application when a client has asked the server to begin a hot link.          *
*                                                                              *
* An instance of this class is created when an instance of IDDETopicServer     *
* needs to pass a beginHotLink request to the server application.  Instances   *
* of this class are not intended to be created by application programmers.     *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from IEvent.                             |
------------------------------------------------------------------------------*/
  IDDEServerHotLinkEvent  ( const IEvent& ddeEvent );

  ~IDDEServerHotLinkEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get the accessible attributes of instances    |
| of this class:                                                               |
|   isPacingRequested - Returns true if the client application has requested   |
|                       the server to request an acknowledgement from the      |
|                       client whenever it sends data or a notification.       |
|   isDataRequested   - Returns true if the client application has requested   |
|                       a data hot link. Returns false if a notification       |
|                       link has been requested.                               |
------------------------------------------------------------------------------*/
Boolean
  isPacingRequested ( ) const,
  isDataRequested   ( ) const;

}; // IDDEServerHotLinkEvent

class IDDEEndEvent : protected IEvent {
typedef IEvent
  Inherited;
/*******************************************************************************
* The IDDEEndEvent class provides event information to a client or server      *
* application when a conversation is ended or ending.                          *
*                                                                              *
* An instance of this class is created when an instance of IDDETopicServer     *
* needs to notify the application that a conversation is ending or ended.      *
* Instances of this class are not intended to be created by application        *
* programmers.                                                                 *
*******************************************************************************/
public:
/*------------------------------ Related Type ----------------------------------
| The following enumeration type defines sources of conversation endings:      |
|   Source   -  Used to defined the possible sources of an end to a            |
|               conversation. The allowable values are:                        |
|     client -  Indicates that the client initiated the conversation's end.    |
|     server -  Indicates that the server initiated the conversation's end.    |
|     error  -  Indicates that an error in a routine entered from the          |
|               dispatcher would cause an exception to be thrown and the       |
|               application would not have an opportunity to catch the         |
|               exception.  Since this situation could cause this              |
|               application and the application being conversed with to        |
|               abnormally end, this application is called back with an        |
|               instance of IDDEEndEvent.  The conversation is also            |
|               terminated at this point.                                      |
------------------------------------------------------------------------------*/
enum Source { client, server, error };

/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from IEvent.                             |
------------------------------------------------------------------------------*/
  IDDEEndEvent  ( const IEvent& ddeEvent,
                  Source        endSource );

  ~IDDEEndEvent ( );

/*-------------------------------- Accessor ------------------------------------
| Use the following function to get the accessible attribute of instances of   |
| this class:                                                                  |
|   sourceOfEnd - Returns one of the values of the Source enumeration          |
|                 indicating what the source of the conversation ending is.    |
------------------------------------------------------------------------------*/
Source
  sourceOfEnd ( ) const;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
Source
  sourceCl;

}; // IDDEEndEvent

class IDDEClientEndEvent : public IDDEEndEvent {
typedef IDDEEndEvent
  Inherited;
/*******************************************************************************
* The IDDEClientEndEvent class provides event information to a client          *
* application when a conversation is ended or ending.                          *
*                                                                              *
* An instance of this class is created when an instance of                     *
* IDDEClientConversation needs to notify the application that a conversation   *
* is ending or ended.  Instances of this class are not intended to be created  *
* by application programmers.                                                  *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from IEvent.                             |
------------------------------------------------------------------------------*/
  IDDEClientEndEvent  ( const IEvent&  ddeEvent,
                        Source         endSource,
                        IString        application,
                        IString        topic );

  ~IDDEClientEndEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following function to get the accessible attributes of this class:   |
|   application - Returns the name of the server application for the           |
|                 conversation that is ending or ended.                        |
|   topic       - Returns the name of the topic supported by the conversation  |
|                 that is ending or ended.                                     |
------------------------------------------------------------------------------*/
IString
  application ( ) const,
  topic       ( ) const;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
IString
  strClApp,
  strClTopic;

}; // IDDEClientEndEvent

class IDDEBeginEvent : protected IEvent {
typedef IEvent
  Inherited;
/*******************************************************************************
* The IDDEBeginEvent class provides event information to a server              *
* application when a client has asked the server to begin a conversation.      *
*                                                                              *
* An instance of this class is created when an instance of                     *
* IDDETopicServer needs to pass an acceptConversation request to the server    *
* application.  Instances of this class are not intended to be created by      *
* application programmers.                                                     *
*******************************************************************************/
public:
/*------------------------- Constructor ----------------------------------------
| Instances of this class are created from an IEvent.                          |
------------------------------------------------------------------------------*/
  IDDEBeginEvent  ( const IEvent& ddeBeginEvent );

  ~IDDEBeginEvent ( );

/*-------------------------------- Accessors -----------------------------------
| Use the following functions to get and set the                               |
| accessible attributes of instances of this class:                            |
|   application      - Returns the name of the requested application.          |
|   topic            - Returns the name of the requested topic.                |
|   setCaseSensitive - Sets the case-sensitive flag to true.  The server       |
|                      application should set this flag if it enforces case    |
|                      sensitivity.                                            |
------------------------------------------------------------------------------*/
IString
  application ( ) const,
  topic       ( ) const;

IDDEBeginEvent
 &setCaseSensitive ( Boolean caseSensitive );

private:
/*--------------------------------- PRIVATE ----------------------------------*/
friend class IDDEClientConversation;
friend class IDDETopicServer;

  IDDEBeginEvent ( const IDDEBeginEvent& rhs );
IDDEBeginEvent&
  operator= ( const IDDEBeginEvent& rhs );
Boolean
  isCaseSensitive ( ) const;

IString
  strClApp,
  strClTopic;
Boolean
  bClCaseSens;

}; // IDDEBeginEvent

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*--------------------------------- INLINES ----------------------------------*/
#ifndef I_NO_INLINES
  #include <iddeevt.inl>
#endif

#endif /* _IDDEEVT_ */
