#ifndef _ILISTBOX_
  #define _ILISTBOX_
/*******************************************************************************
* FILE NAME: ilistbox.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IListBox - This class creates and manages the listbox control window.    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif
// Forward declarations for other classes:
class IString;
class IResourceId;
#ifndef _IRECT_
  #include <irect.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

/*----------------------------------------------------------------------------*/
/* Turn off warning for compiler generated copy/assignment                    */
/*----------------------------------------------------------------------------*/
#pragma info(nocpy)

class IListBox : public IControl  {
typedef IControl Inherited;
/*******************************************************************************
* The IListBox class creates and manages the list box control window.  A list  *
* box operates as if it is a 0-based array of items.  The item index           *
* requested or returned is the 0-based index number of the location of the     *
* item in question.                                                            *
*                                                                              *
* EXAMPLE:                                                                     *
*  #include <ilistbox.hpp>                                                     *
*  lb_test = new IListBox(ID_LISTBOX, this, this, IRectangle(x1, y1, x2, y2)); *
*  lb_test->addItemAscending("This is first item");                            *
*  SelectedItem = lb_test->selection();                                        *
*******************************************************************************/
public:
/*---------------------- Style --------------------------------------------
  The following functions provide a means to set and query list box styles:

    Style - Nested class that provides static members that define the set of
            valid list box styles.  These styles can be used in conjunction
            with the styles defined by the nested classes IWindow::Style and
            IControl::Style.  For example, you could define an instance of
            the IListBox::Style class and initialize it like:
              IListBox::Style
                style = IListBox::horizontalScroll | IControl::tabStop;
            An object of this type is provided when the list box is created.
            A customizable default is used if no styles are specified.  Once
            the object is constructed, IListBox, IWindow, and IControl
            member functions can be used to set or query the object's style.

            The declaration of the IListBox::Style nested class is generated
            by the INESTEDBITFLAGCLASSDEF2 macro.

  The valid list box styles are:
    classDefaultStyle - This specifies the original default style for this
                        class, which is horizontalScroll | noAdjustPosition |
                        IWindow::visible.
    horizontalScroll  - This style allows horizontal scrolling of the list
                        box.
    multipleSelect    - This style specifies that the user can select any
                        number of objects at a time in the list box, or not
                        select any.
    extendedSelect    - This style is a type of selection optimized for the
                        selection of a single object.  A user can extend
                        selection to more than one object, if required.
    drawItem          - This style allows list box items to be drawn.  It is
                        typically used to display bit maps.
    noAdjustPosition  - If this style is included, the list box control is
                        drawn at the size specified. This can cause an item
                        to be only partially shown.

  The following functions provide a means of getting and setting the default
  style for this class:
    defaultStyle    - Returns the current default style.  This is the same as
                      classDefaultStyle unless setDefaultStyle has been
                      called.
    setDefaultStyle - Sets the default style for all subsequent list boxes.
-------------------------------------------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, IListBox, IWindow, IControl);

static const Style
  classDefaultStyle,
  horizontalScroll,
  multipleSelect,
  extendedSelect,
  drawItem,
  noAdjustPosition;

static Style
   defaultStyle();
static void
   setDefaultStyle(Style style);

/*------------------------ Constructors ----------------------------------------
| You can construct an instance of this class in the following ways:           |
|    - From a control ID, parent and owner windows, rectangle, and style.      |
|      This creates the specified list box control and an object for it.       |
|    - From the ID of a list box control on a dialog window.  This creates     |
|      the object for the specified list box control.                          |
|    - From the window handle of an existing list box control.  This creates   |
|      the object for the specified list box control.                          |
------------------------------------------------------------------------------*/
  IListBox(unsigned long id,
           IWindow* parent,
           IWindow* owner,
           const IRectangle& initial= IRectangle(),
           const Style& style = defaultStyle() );

  IListBox(unsigned long id,
           IWindow* parent);

  IListBox(const IWindowHandle& handle);

  virtual ~IListBox();

/*------------------------ Add Item Operations ---------------------------------
| Add a line of text to the list box.                                          |
|   add          - Allows the line to be inserted any place in the list box.   |
|   addAsFirst   - Allows the line to be inserted at the top of the list box.  |
|   addAsLast    - Allows the line to be inserted to the bottom of the list    |
|                  box.                                                        |
|   addAscending - Allows the line to be inserted in ascending sort order.     |
|   addDescending- Allows the line to be inserted in descending sort order.    |
------------------------------------------------------------------------------*/
unsigned long
  add(unsigned long  index, const char* text),
  addAsFirst(const char* text),
  addAsLast(const char* text),
  addAscending(const char* text),
  addDescending(const char* text),
  add(unsigned long  index, const IResourceId& resid),
  addAsFirst(const IResourceId& resid),
  addAsLast(const IResourceId& resid),
  addAscending(const IResourceId& resid),
  addDescending(const IResourceId& resid);

/*------------------------ Get/Change Item Operations --------------------------
| Get or change the text of an item in the list box.                           |
|   itemText    - Returns the text of an item in the list box                  |
|   setItemText - Changes the text of an item in the list box.                 |
------------------------------------------------------------------------------*/
IString
  itemText(unsigned long index) const;
IListBox&
  setItemText(unsigned long index, const char* string);

/*------------------------ Remove Item Operations ------------------------------
| Remove all or one item from the list box.                                    |
|   removeAll - Removes all items from the list box.                           |
|   remove    - Removes one item from the list box and returns the count of    |
|               the number of items that remain.                               |
------------------------------------------------------------------------------*/
IListBox&
  removeAll();
long
  remove(unsigned long index );

/*------------------------ Selection-state Operations --------------------------
| Set or return the selection state operations.                                |
|   select             - Sets the selection state of an item based on the      |
|                        Boolean value that is passed in.  If true is passed   |
|                        in and the list box is a single-selection list box,   |
|                        the item is selected and any item that was            |
|                        previously selected is deselected.  If true is        |
|                        passed in and the list box is a multiple-selection    |
|                        or extended-selection list box, the item is selected. |
|                        In all cases, if false is passed in, the item is      |
|                        deselected.                                           |
|   deselect           - Removes the selection state from an item.             |
|   selectAll          - Sets the selection state for all items in the list    |
|                        box.                                                  |
|   deselectAll        - Removes the selection state from all items in the     |
|                        list box.                                             |
|   isSelected         - Returns the selection state of an item.               |
|   numberOfSelections - Returns 0 if no item in a single-selection list box   |
|                        is selected or 1 if an item is selected.  Returns     |
|                        the number of selected items in a multiple-selection  |
|                        or extended-selection list box.  In all cases, the    |
|                        current selection does not change.                    |
|   selection          - Returns the 0-based index of the selected item in a   |
|                        single-selection list box, or the 0-based index of    |
|                        the first selected item in a multiple-selection or    |
|                        extended-selection list box.  In all cases, if no     |
|                        item is selected, -1 is returned.                     |
------------------------------------------------------------------------------*/
IListBox
  &select(unsigned long index, Boolean select = true ),
  &deselect(unsigned long index ),
  &selectAll(),
  &deselectAll();
Boolean
  isSelected(unsigned long index );
unsigned long
  numberOfSelections() const;
long
  selection();

/*------------------------ Scrolling Operations --------------------------------
| Scrolls an item to the top of the list box or returns the item at the top of |
| the list box.                                                                |
|   setTop - Scrolls the requested item to the top of the list box.            |
|   top    - Returns the item number of the item currently at the top of the   |
|            list box. If there are no items in the list box, an exception is  |
|            thrown.                                                           |
------------------------------------------------------------------------------*/
IListBox&
  setTop(unsigned long index);
unsigned long
  top() const;

/*------------------------ Handle Operations -----------------------------------
| Set or retrieve a handle of a list box item.                                 |
|   setItemHandle - Sets a handle of a list box item.                          |
|   itemHandle    - Returns a handle of a list box item.  An item without a    |
|                   handle returns 0.                                          |
------------------------------------------------------------------------------*/
IListBox&
  setItemHandle(unsigned long index, unsigned long handle );
unsigned long
  itemHandle(unsigned long index ) const;

/*---------------------------- Enumerations ------------------------------------
| The following enumerations are defined:                                      |
|   SearchType - Used for determining the type of search to perform.  Values   |
|               are:                                                           |
|               prefix      - Matching occurs if the leading characters        |
|                             of the item contain the specified characters.    |
|               substring   - Matching occurs if the item contains a substring |
|                             of the specified characters.                     |
|               exactMatch  - Matching occurs if the item is an exact match for|
|                             the specified characters.                        |
------------------------------------------------------------------------------*/
enum SearchType {
  prefix,
  substring,
  exactMatch
};

/*------------------------ Query Operations ------------------------------------
| locateText - Returns the item number of the list box item that matches the   |
|              search string.  The search is started after the index.  If no   |
|              match is found, it returns notFound.  If there is an error, an  |
|              exception occurs.                                               |
------------------------------------------------------------------------------*/
static const unsigned long notFound ;
static const long first ;
unsigned long
  locateText(const char* searchString,
             Boolean caseSensitive = true,
             SearchType search = exactMatch,
             long  index = first) const;

/*------------------------ Miscellaneous Operations ----------------------------
| count     - Returns the count of the number of items in the list box.        |
| setHeight - Sets the height of a list box item, in pixels.  This does not    |
|             change the font size.  It changes the height of the cell within  |
|             which an item is displayed.                                      |
------------------------------------------------------------------------------*/
unsigned long
  count() const;
IListBox&
  setHeight(long newHeight);

/*---------------------------- Enumerations ------------------------------------
| The following enumerations are defined:                                      |
|   ColorArea - Used to replace the color for a particular region.  Values     |
|               are:                                                           |
|               foreground          - Sets the color for the foreground text.  |
|               disabledForeground  - Sets the foreground color for disabled   |
|                                     text.                                    |
|               highlightForeground - Sets the foreground color for            |
|                                     highlighted text.                        |
|               border              - Sets the color of the border that        |
|                                     surrounds the list box.                  |
------------------------------------------------------------------------------*/
enum ColorArea {
  foreground,
  disabledForeground,
  highlightForeground,
  border
};
/*----------------------------- Color Functions --------------------------------
|   setColor   - Changes the color of the displayed text.                      |
|   color      - Returns the color of the displayed text.                      |
------------------------------------------------------------------------------*/

IListBox
  &setColor(ColorArea value, const IColor& color);

IColor
  color(ColorArea value) const;

/*------------------------ Style Functions -------------------------------------
| isHorizontalScroll      - Returns true if the horizontalScroll style is set. |
| isMultipleSelect        - Returns true if the multipleSelect style is set.   |
| isExtendedSelect        - Returns true if the extendedSelect style is set.   |
| isDrawItem              - Returns true if the drawItem style is set.         |
| isNoAdjustPosition      - Returns true if the noAdjustPosition style is set. |
| enableMultipleSelect    - Enables or disables the multipleSelect style for   |
|                           the list box.                                      |
| disableMultipleSelect   - Disables the multipleSelect style for the list     |
|                           box.                                               |
| enableExtendedSelect    - Enables or disables the extendedSelect style for   |
|                           the list box.                                      |
| disableExtendedSelect   - Disables the extendedSelect style for the list     |
|                           box.                                               |
| enableDrawItem          - Enables or disables the drawItem style for the     |
|                           list box.                                          |
| disableDrawItem         - Disables the drawItem style for the list box.      |
| enableNoAdjustPosition  - Enables or disables the noAdjustPosition style     |
|                           for the list box.                                  |
| disableNoAdjustPosition - Disables the noAdjustPosition style for the list   |
|                           box.                                               |
------------------------------------------------------------------------------*/
Boolean isHorizontalScroll(),
  isMultipleSelect(),
  isExtendedSelect(),
  isDrawItem(),
  isNoAdjustPosition();
IListBox
  &enableMultipleSelect(Boolean enable = true),
  &disableMultipleSelect(),
  &enableExtendedSelect(Boolean enable = true),
  &disableExtendedSelect(),
  &enableDrawItem(Boolean enable = true),
  &disableDrawItem(),
  &enableNoAdjustPosition(Boolean enable = true),
  &disableNoAdjustPosition();

class Cursor : public IVBase {
/*******************************************************************************
* The IListBox::Cursor class creates and manages the cursor support of the     *
* list box.                                                                    *
*                                                                              *
* EXAMPLE:                                                                     *
*  #include <ilistbox.hpp>                                                     *
*  lb_test = new IListBox(ID_LISTBOX, this, this, IRectangle(x1, y1, x2, y2)); *
*  lb_test->addAscending("This is first item");                                *
*  lb_cursor = new IListBox::Cursor(*lb_test);                                 *
*  lb_cursor->setToFirst();                                                    *
*  CurrentItem = lb_cursor->asIndex();                                         *
*******************************************************************************/
  public :
  /*------------------------ Filters -------------------------------+
  | The following functions allow you to specify how list box items |
  | will be filtered:                                               |
  |   Filter - Enumeration that defines which items are to be       |
  |            filtered (selectedItems or allItems).                |
  +----------------------------------------------------------------*/
  enum Filter { selectedItems, allItems };
  /*------------------------ Constructors ---------------------------
  | You can create instances of this class from a list box control  |
  | and a filter type.  This creates the specified list box cursor  |
  | object.                                                         |
  -----------------------------------------------------------------*/
  Cursor(const IListBox& rlb,
         Filter type = selectedItems);

  virtual ~Cursor();

  /*------------------------ Cursor Movement -----------------------+
  | setToFirst    - Points to the list box item and validates the   |
  |                 cursor.                                         |
  | setToNext     - Points to the next item in the list box.  If no |
  |                 more items exist, this will invalidate the      |
  |                 cursor.                                         |
  | setToPrevious - Points to the previous item in the list box.    |
  |                 If no previous items exist, this will           |
  |                 invalidate the cursor.                          |
  | setToLast     - Points to the last list box item and validates  |
  |                 the cursor.                                     |
  | setToIndex    - Points to the item with the given 0-based index |
  |                 and validates the cursor.                       |
  +----------------------------------------------------------------*/
  Boolean
    setToFirst(),
    setToNext(),
    setToPrevious(),
    setToLast(),
    setToIndex(unsigned long index);

  /*--------------- Cursor Validation/Conversion -------------------+
  | isValid    - Queries whether this cursor points to a valid      |
  |              item.                                              |
  | invalidate - Flags this cursor as invalid.                      |
  | asIndex    - Returns the index of the item to which the cursor  |
  |              is pointing.                                       |
  +----------------------------------------------------------------*/
  Boolean
    isValid() const;
  void
    invalidate();
  unsigned long
    asIndex() const;

  private :
  const IListBox&
    rlbCl;
  unsigned long
    lClCurrent;
  Filter
    cursorTypeCl;
  unsigned long
    sameValidation;
};
/*------------------------ Cursor Operations ---------------------+
| The following are operations on list box items using a cursor:  |
|   elementAt   - Returns the item's string at the cursor.        |
|   removeAt    - Removes the item at the cursor and places the   |
|                 cursor at the next available valid item.        |
|   replaceAt   - Replaces the item's string at the cursor.       |
|   add         - Adds the item at the cursor by pushing          |
|                 everything after the cursor.                    |
|   addAsFirst  - Adds the item as the first item in the list box |
|                 and sets the cursor on the added item.          |
|   addAsLast   - Adds the item as the last item in the list box  |
|                 and sets the cursor on the added item.          |
|   addAsNext   - Adds the item as the next item in the list box  |
|                 and sets the cursor on the added item.          |            |
+----------------------------------------------------------------*/
IString
  elementAt(const Cursor& cursor) const;
IListBox
  &add(const char* strItem, Cursor& cursor),
  &add(const IResourceId& resid  , Cursor& cursor),
  &removeAt(Cursor& cursor),
  &replaceAt(const char* strNew,const Cursor& cursor),
  &addAsFirst(const char* strItem, Cursor& cursor),
  &addAsLast(const char* strItem, Cursor& cursor),
  &addAsNext(const char* strItem, Cursor& cursor),
  &addAsFirst(const IResourceId& resid, Cursor& cursor),
  &addAsLast( const IResourceId& resid, Cursor& cursor),
  &addAsNext( const IResourceId& resid, Cursor& cursor),
  &replaceAt( const IResourceId& resid, const Cursor& cursor);

/*----------------------- ISet Operations ------------------------+
| The following operates on list boxes with the same terminology  |
| as BBXX ISet:                                                   |
|   isEmpty - Returns true if the list box is empty.              |
+----------------------------------------------------------------*/
Boolean
  isEmpty() const;

protected:

private:
static  Style
  currentDefaultStyle;
unsigned long
  needValidation;
friend class IListBox::Cursor;

};   //ILISTBOX


//Global Style Functions
INESTEDBITFLAGCLASSFUNCS(Style, IListBox);

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing and warning messages.                      */
/*----------------------------------------------------------------------------*/
#pragma pack()
#pragma info(restore)

#endif  /* _ILISTBOX_ */
