#ifndef _IMLE_
  #define _IMLE_
/*******************************************************************************
* FILE NAME: imle.hpp                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IMultiLineEdit - This class creates and manages the multiple-line entry  *
*                      field control.                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ITEXTCTL_
  #include <itextctl.hpp>
#endif

// Forward declarations for other classes:
class IWindowHandle;
class IFont;
class IColor;
#ifndef _IRECT_
  #include <irect.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

/*----------------------------------------------------------------------------*/
/* Turn off warning for compiler generated copy/assignment                    */
/*----------------------------------------------------------------------------*/
#pragma info(nocpy)

class IMultiLineEdit : public ITextControl {
typedef ITextControl Inherited;
/*******************************************************************************
* The IMultiLineEdit class creates and manages the multiple-line entry field   *
* (MLE) control.                                                               *
*                                                                              *
*                                                                              *
* EXAMPLE:                                                                     *
*   #include <imle.hpp>                                                        *
*   IString pchInitText[100];                                                  *
*   pmle = new IMultiLineEdit( ID_MLE, this, this,                             *
*                              IRectangle(IPoint(50, 15),                      *
*                                         IPoint(200, 200)),                   *
*                              IMultiLineEdit::classDefaultStyle );            *
*   strcpy(pchInitText, "Text init'd by setText, selected by selectRange.");   *
*   pmle->setText( pchInitText, strlen(pchInitText)+1);                        *
*   pmle->selectRange();                                                       *
*   pmle->setFocus();   // must setFocus to show marked area                   *
*******************************************************************************/
public:

/*---------------- Style -----------------------------------------------------
  The following functions provide a means to set and query MLE styles:

    Style - Nested class that provides static members that define the set of
            valid MLE styles.  These styles can be used in conjunction with
            the styles defined by the nested classes IWindow::Style and
            IControl::Style.  For example, you could define an instance of
            the IMultiLineEdit::Style class and initialize it like:
              IMultiLIneEdit::Style
                style = IMultiLineEdit::border | IControl::tabStop;
            An object of this type is provided when the MLE is created.  A
            customizable default is used if no styles are specified.  Once
            the object is constructed, IMultiLineEdit, IWindow, and IControl
            member functions can be used to set or query the object's style.

            The declaration of the IMultiLineEdit::Style nested class is
            generated by the INESTEDBITFLAGCLASSDEF2 macro.

  The valid MLE styles are:
    classDefaultStyle - Original default style for this class.  This style is
                        border | vertScroll | wordWrap | autoScroll |
                        IWindow::visible.
    border            - Causes a thin border to be drawn around the MLE.
    readOnly          - Restricts the user from entering any data.
    wordWrap          - Causes text to word-wrap at the end of a line.
    horizontalScroll  - Adds a horizontal scroll bar to the MLE.
    verticalScroll    - Adds a vertical scroll bar to the MLE.
    ignoreTab         - Causes the MLE to ignore tab key strokes.  This
                        function sends a keyboard event to the owner of the
                        MLE.

  The following functions provide a means of getting and setting the default
  style for this class:
    defaultStyle    - Returns the current default style.  This is the same as
                      classDefaultStyle unless setDefaultStyle has been
                      called.
    setDefaultStyle - Sets the default style for all subsequent MLEs.
-------------------------------------------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, IMultiLineEdit, IWindow, IControl);

static const Style
  classDefaultStyle,
  border,
  readOnly,
  wordWrap,
  horizontalScroll,
  verticalScroll,
  ignoreTab;

static Style
  defaultStyle();

static void
  setDefaultStyle(Style style);

/*------------------------ Constructors ----------------------------------------
| You can construct an instance of this class in the following ways:           |
|   - The specified multiple-line entry field control and an object for it     |
|     is created from a control ID, parent and owner windows, rectangle, and   |
|     style.                                                                   |
|   - The object for the specified multiple-line entry field control is        |
|     created from the ID of a multiple-line entry field control on a dialog   |
|     window.                                                                  |
|   - The object for the specified multiple-line entry field control is        |
|     created from the window handle of an existing multiple-line entry        |
|     field control.                                                           |
|                                                                              |
| NOTE: You can type past the text limit of an IMultiLineEdit window that is   |
|       constructed from direct editing in a container.                        |
------------------------------------------------------------------------------*/
  IMultiLineEdit(unsigned long id,
                 IWindow* parent,
                 IWindow* owner,
                 const IRectangle& initial= IRectangle(),
                 const Style& style = defaultStyle() );
  IMultiLineEdit(unsigned long id,
                 IWindow* parent);
  IMultiLineEdit(const IWindowHandle& handle);

  virtual ~IMultiLineEdit();

/*-------------------------------- Change Operations ---------------------------
| setChangedFlag    - Sets a flag to indicate that the MLE contents have       |
|                     changed.                                                 |
| removeChangedFlag - Resets the changed flag so that changes to the MLE from  |
|                     this point forward can be detected.                      |
| isChanged         - Queries whether any changes have been made to the MLE    |
|                     since the last time the changed flag was reset.          |
------------------------------------------------------------------------------*/
IMultiLineEdit
  &setChangedFlag(),
  &removeChangedFlag();
Boolean
  isChanged() const;

/*-------------------------------- Read-only Operations ------------------------
|  disableDataUpdate - Prohibits changes to the current MLE text.              |
|  enableDataUpdate  - Sets the MLE to read-write mode, which allows the user  |
|                      to update the MLE text.                                 |
|  isReadOnly        - Queries whether the MLE is in read-only mode.           |
------------------------------------------------------------------------------*/
IMultiLineEdit
  &disableDataUpdate(),
  &enableDataUpdate(Boolean update=true);
Boolean
  isReadOnly() const;

/*-------------------------------- Refresh Operations --------------------------
|  enableRefresh  - Enables screen refreshes.  The screen is updated with the  |
|                   contents of the text.                                      |
|  disableRefresh - Disables screen refreshes.  Allows an application to make  |
|                   changes throughout an MLE while avoiding unnecessary       |
|                   overhead caused by attempts to keep the screen display     |
|                   updated.  While refresh is disabled, mouse and keyboard    |
|                   messages are processed by beeping and ignoring them.  The  |
|                   mouse pointer changes to the system standard "wait"        |
|                   pointer.                                                   |
------------------------------------------------------------------------------*/
IMultiLineEdit
  &enableRefresh(Boolean refresh=true),
  &disableRefresh();

/*-------------------------------- Text Operations -----------------------------
|  EOLFormat   - Enumerates the formats for the end-of-line characters for     |
|                importing and exporting.  Valid formats are:                  |
|                  cfText    - Each line ends with a carriage-return/line-     |
|                              feed combination.  Tab characters separate      |
|                              fields within a line.  A NULL character         |
|                              signals the end of the data.                    |
|                  noTran    - Uses LF for line delineation and guarantees     |
|                              any text imported into the MLE in this format   |
|                              can be recovered in exactly the same form when  |
|                              exported.                                       |
|                  MLEFormat - On import, recognizes CRLF as denoting hard     |
|                              line-breaks, and ignores the sequence CRCRLF.   |
|                              On export, uses CRLF to denote a hard line-     |
|                              break and CRCRLF to denote a soft line-break    |
|                              caused by word-wrapping.                        |
|  setText     - Overwrites the current contents of the MLE with the new text  |
|                contained in the input buffer.  The input buffer can contain  |
|                NULL characters; the bufferSize argument determines how much  |
|                text is copied into the MLE.                                  |
|  text        - If no parameter is specified, it returns the current          |
|                contents of the MLE.  If a line number is specified, it       |
|                returns the current MLE text at the given line.               |
|  setLimit    - Sets a limit on the number of bytes that can be input into    |
|                the MLE.  If the MLE already contains more bytes than the     |
|                proposed limit, an exception is thrown.                       |
|  limit       - Returns the currently set number of bytes the MLE can hold.   |
|  textLength  - Returns the current length of the MLE text, in bytes.  The    |
|                length includes any CRLFs present.                            |
|  addAtOffset - Inserts the specified text into the MLE at the point          |
|                specified.  At the conclusion of this insert, the cursor      |
|                will be positioned at the same character as it was before     |
|                the insert.  The textSize argument is used in case you want   |
|                want to add text that contains more than one NULL character.  |
|                If textSize is not specified, the function will add the       |
|                characters up to the first NULL character.                    |
|  add         - Inserts the specified text into the MLE at the current        |
|                cursor position.  At the conclusion of this operation, the    |
|                cursor will be positioned at the end of the inserted text.    |
|                The textSize argument is used in case you want to add text    |
|                that contains more than one NULL character.  If textSize is   |
|                not specified, the function will add the characters up to     |
|                the first NULL character.                                     |
|  addAsLast   - Inserts the specified text into the MLE at the end of the     |
|                current text.  This operation will not change the cursor      |
|                position.  The textSize argument is used in case you want to  |
|                add text that contains more than one NULL character.  If      |
|                textSize is not specified, the function will add the          |
|                characters up to the first NULL character.                    |
------------------------------------------------------------------------------*/
enum EOLFormat {cfText,noTran,MLEFormat};
IString
  text() const,
  text( unsigned long lineNumber) const;
unsigned long
  limit() const;
virtual unsigned long
  textLength() const;
IMultiLineEdit
  &setText(const char* text),
  &setText(const IResourceId& text),
  &setText(char* buffer,
          unsigned long bufferSize ),
  &setLimit(unsigned long newLimit),
  &add(char* text, unsigned long textSize=0, EOLFormat type=cfText),
  &addAsLast(char* text, unsigned long textSize=0,
             EOLFormat type=cfText),
  &addAtOffset(char* text, unsigned long charnumber,
               unsigned long textSize=0,  EOLFormat type=cfText);

/*-------------------------------- Import/Export Operations --------------------
|  importFromFile           - Inserts the contents of the specified file into  |
|                             the MLE at the current cursor position and       |
|                             returns the number of characters imported.       |
|  exportToFile             - Saves the contents of the MLE to the specified   |
|                             file and returns the number of bytes written to  |
|                             the file.                                        |
|  exportSelectedTextToFile - Saves the currently marked text to the           |
|                             specified file and returns the number of bytes   |
|                             written to the file.                             |
------------------------------------------------------------------------------*/
unsigned long
  importFromFile(char* fileName, EOLFormat type=cfText),
  exportToFile(char * fileName, EOLFormat type=cfText),
  exportSelectedTextToFile(char* fileName, EOLFormat type=cfText);

/*-------------------------------- Selected Text -------------------------------
|   end                - Used to denote the end of the text for selecting      |
|                        text.                                                 |
|   selectRange        - Selects a range of text.  The selected range is the   |
|                        0-based index of the first character selected (or to  |
|                        be selected) and the 0-based index of the last        |
|                        character selected (or to be selected).  If no range  |
|                        is specified, all of the text will be selected.       |
|                        Specifying a range where the lower and upper values   |
|                        are equal will cause the selection to become an       |
|                        insertion point.                                      |
|   selectedRange      - Returns the range of the selected text. If no text    |
|                        is selected, a range of (cursor position, cursor      |
|                        position) is returned by selectedRange.  The          |
|                        selected range is the 0-based index of the first      |
|                        character being selected and the 0-based index of     |
|                        the last character being selected.                    |
|   selectedText       - Returns the selected text string.                     |
|   selectedTextLength - Returns the size of the selected area, in bytes.      |
|                        The length includes CRLFs, but not NULL terminators.  |
|   hasSelectedText    - Queries whether any of the MLE's text is selected.    |
------------------------------------------------------------------------------*/
static const long end ;
IMultiLineEdit&
  selectRange(const IRange& range=IRange(0,end));
IRange
  selectedRange() const;
IString
  selectedText() const;
unsigned long
  selectedTextLength() const;
Boolean
  hasSelectedText() const;
/*-------------------------------- Clipboard Operations ------------------------
|  cut     - Copies the currently selected text from the MLE to the clipboard  |
|            and then deletes the selected text from the MLE.                  |
|  copy    - Copies the currently selected text from the MLE to the clipboard. |
|  paste   - Inserts the contents of the clipboard into the MLE at the         |
|            current cursor position.                                          |
|  clear   - Replaces the currently selected text with blank spaces.           |
|  discard - Deletes all of the currently selected text.                       |
------------------------------------------------------------------------------*/
virtual IMultiLineEdit
  &cut(),
  &copy(),
  &paste(),
  &clear(),
  &discard();

/*-------------------------------- Undo Operations -----------------------------
|  isUndoable - Queries whether any undoable actions have been performed on    |
|               the contents of the MLE.                                       |
|  undo       - Restores the MLE contents to the state they were in before     |
|               the last change.                                               |
------------------------------------------------------------------------------*/
Boolean
  isUndoable() const;
IMultiLineEdit&
  undo();

/*-------------------------------- Delete Text Operation -----------------------
|  removeAll - Deletes the entire contents of the MLE.                         |
------------------------------------------------------------------------------*/
IMultiLineEdit&
  removeAll();

/*-------------------------------- Size Operations -----------------------------
|  setEditRegion       - Sets the size of the edit region so that it covers    |
|                        the entire MLE window.  The edit region is the        |
|                        rectangular area in which the text is displayed.      |
|  setEditRegionWidth  - Sets the width of the edit region, in pixels.         |
|  setEditRegionHeight - Sets the height of the edit region, in pixels.        |
|  editRegionWidth     - Returns the width of the edit region.                 |
|  editRegionHeight    - Returns the height of the edit region.                |
------------------------------------------------------------------------------*/
IMultiLineEdit
  &setEditRegion(),
  &setEditRegion( const ISize& sizeEditRegion),
  &setEditRegionWidth( long width),
  &setEditRegionHeight( long height);
unsigned long
  editRegionWidth()  const,
  editRegionHeight() const;

/*-------------------------------- Tab Stop Operation --------------------------
|  setTab - Sets the number of pixels between tab stops inside the MLE.        |
------------------------------------------------------------------------------*/
IMultiLineEdit&
  setTab(unsigned long tabPixelInterval);

/*-------------------------------- Word-wrap Operations ------------------------
|  isWordWrap      - Queries whether the MLE is in word-wrap mode.             |
|  enableWordWrap  - Enables word-wrap mode in the MLE.                        |
|  disableWordWrap - Disables word-wrap mode in the MLE.                       |
------------------------------------------------------------------------------*/
Boolean
  isWordWrap() const;
IMultiLineEdit
  &enableWordWrap(Boolean enable=true),
  &disableWordWrap();

/*---------------------------- Enumerations ------------------------------------
| The following enumeration is defined:                                        |
|   ColorArea - Enumeration that is used to replace the color for a            |
|               particular region.  Values are:                                |
|               foreground  - Sets the color of the foreground text.           |
|               background  - Sets the color of the background for the text.   |
|               bordercolor - Sets the color of the border that surrounds the  |
|                             MLE window.                                      |
------------------------------------------------------------------------------*/
enum ColorArea {
  foreground,
  background,
  bordercolor
};

/*------------------------ Color and Font Functions ----------------------------
| These functions provide a means to change and query string attributes of     |
| instances of this class:                                                     |
|  setColor - Changes the color of the specified color area.                   |
|  color    - Returns the color of the specified color area.                   |
|  setFont  - Changes the font of the displayed text.                          |
------------------------------------------------------------------------------*/
virtual IMultiLineEdit
  &setColor(ColorArea value, const IColor& color),
  &setFont(const IFont& font);

IColor
  color(ColorArea value) const;

/*-------------------------------- Line Operations -----------------------------
| These functions use the terms line and line number as arguments or returns   |
| from these functions.  A line is defined as a line on the display after the  |
| application of word-wrap.  It does not mean a line as defined by the CRLF    |
| line-break sequence.                                                         |
|  addLine         - Inserts the specified NULL-terminated text at the         |
|                    specified line number.                                    |
|  addLineAsLast   - Inserts the specified NULL-terminated text at the last    |
|                    line.                                                     |
|  removeLine      - Deletes the specified line of text.                       |
|  setTop          - Makes the specified line the topmost visible line on the  |
|                    screen.                                                   |
|  setCursorAtLine - Moves the cursor position to the specified line number.   |
|  setCursorAt     - Moves the cursor a specified number of positions from     |
|                    left to right in the MLE.  The count begins with the      |
|                    first position in the MLE, not the cursor's current       |
|                    position.                                                 |
|  top             - Returns the line number of the line currently visible at  |
|                    the top of the screen.                                    |
|  cursor          - Returns the line number of the line that currently        |
|                    contains the cursor.                                      |
|  numberOfLines   - Returns the total number of lines of text in the MLE.     |
|  visibleLines    - Returns the number of lines that completely fit inside    |
|                    the edit region of the MLE based on the current font.     |
------------------------------------------------------------------------------*/
IMultiLineEdit
  &addLine(char* text, unsigned long lineNumber, EOLFormat type=cfText),
  &addLineAsLast(char* text, EOLFormat type=cfText),
  &removeLine(unsigned long lineNumber),
  &setTop(unsigned long lineNumber),
  &setCursorAtLine(unsigned long lineNumber),
  &setCursorAt(unsigned long cursorPosition);
unsigned long
  top() const,
  cursor() const,
  numberOfLines() const,
  visibleLines() const;


private:
/*--------------------------------- Private ----------------------------------*/
static Style
  currentDefaultStyle;
long
  exportFile(char *fileName, Boolean flMarkedAreaOnly);
unsigned long
  export(long *insertion_point,unsigned long *buffer_length),
  import(long *insertion_point,unsigned long buffer_length);
IMultiLineEdit&
  setIEBuffer(char *buffer,unsigned long length);
Boolean
  bRefresh;
unsigned long
  textLengthAfterFormat() const;
} ;

INESTEDBITFLAGCLASSFUNCS(Style, IMultiLineEdit);

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing and warning messages.                      */
/*----------------------------------------------------------------------------*/
#pragma pack()
#pragma info(restore)

#endif
