#ifndef _INOTEBK_
  #define _INOTEBK_
/*******************************************************************************
* FILE NAME: inotebk.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IPageHandle - This class represents a page in a notebook control.        *
*     INotebook - This class creates and manages the notebook control window.  *
*     INotebook::Cursor - This class is a search cursor for notebook pages.    *
*     INotebook::PageSettings - Contains information for a page in a notebook  *
*                               or for a page to be added to a notebook.       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Forward declarations for other classes:
class IResourceId;
class ITabBitmapMgr;

#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif
#ifndef _IRECT_
  #include <irect.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

/*----------------------------------------------------------------------------*/
/* Turn off warning for compiler generated copy/assignment                    */
/*----------------------------------------------------------------------------*/
#pragma info(nocpy)

class IPageHandle : public IHandle {
typedef IHandle
  Inherited;
/*******************************************************************************
* The IPageHandle class represents a page in a notebook and instances are      *
* only intended to be constructed by a notebook.  An IPageHandle is returned   *
* by a notebook when a page is added to the notebook (the page handle          *
* represents a new page) or when a page is queried (such as from a call to     *
* the INotebook::first function).  You can use an IPageHandle object to        *
* identify a page, just as you can with an INotebook::Cursor cursor object.    *
*******************************************************************************/
public:
/*------------------------------- Constructor ----------------------------------
| Objects of this class are constructed from a page identifier (a value of     |
| type IHandle::Value), defaulting to 0.                                       |
------------------------------------------------------------------------------*/
  IPageHandle ( Value pageId = 0 );

}; // class IPageHandle


class INotebook : public IControl {
typedef IControl
  Inherited;
/*******************************************************************************
* The INotebook class is used to create and manage a notebook control.  Two    *
* classes are scoped to this class.  The scoped PageSettings class is used to  *
* define the characteristics of a page when adding it to a notebook.  The      *
* scoped Cursor class is used to traverse the pages of the notebook.           *
*******************************************************************************/
public:
/*--------------------- Style ---------------------------------------------
  The following functions provide a means to set and query notebook styles:

    Style - Nested class that provides static members that define the set of
            valid notebook styles.  These styles can be used in conjunction
            with the styles defined by the nested classes IWindow::Style and
            IControl::Style.  For example, you could define an instance of
            the INotebook::Style class and initialize it like:
              INotebook::Style
                style = INotebook::roundedTabs | IControl::tabStop;
            An object of this type is provided when the notebook is created.
            A customizable default is used if no styles are specified.  Once
            the object is constructed, INotebook, IWindow, and IControl
            member functions can be used to set or query the object's style.

            The declaration of the INotebook::Style nested class is
            generated by the INESTEDBITFLAGCLASSDEF2 macro.

  The valid notebook styles are:
    classDefaultStyle    - Original default style for this class, which is
                           squareTabs | solidBinding | statusTextLeft |
                           tabTextCenter | backPagesBottomRight |
                           majorTabsRight | IWindow::visible.
    spiralBinding        - A spiral binding is drawn on the notebook.
    solidBinding         - A solid binding is drawn.  This is the default.
    backPagesBottomRight - The back pages of the notebook are drawn behind
                           the bottom right corner of the notebook.  This
                           is the default.
    backPagesBottomLeft  - The back pages of the notebook are drawn behind
                           the bottom left corner of the notebook.
    backPagesTopRight    - The back pages of the notebook are drawn behind
                           the top right corner of the notebook.
    backPagesTopLeft     - The back pages of the notebook are drawn behind
                           the top left corner of the notebook.
    majorTabsRight       - The major tabs of the notebook are drawn on the
                           right side of the notebook. This is the default.
    majorTabsLeft        - The major tabs of the notebook are drawn on the
                           left side of the notebook.
    majorTabsTop         - The major tabs of the notebook are drawn on the
                           top side of the notebook.
    majorTabsBottom      - The major tabs of the notebook are drawn on the
                           bottom side of the notebook.
    squareTabs           - Tabs are drawn with square corners.  This is the
                           default.
    roundedTabs          - Tabs are drawn with rounded corners.
    polygonTabs          - Tabs are drawn with polygon corners.
    statusTextLeft       - The text in the status line is left-justified.
                           This is the default.
    statusTextRight      - The text in the status line is right-justified.
    statusTextCenter     - The text in the status line is centered.
    tabTextCenter        - The text in the tabs is centered.  This is the
                           default.
    tabTextLeft          - The text in the tabs is left-justified.
    tabTextRight         - The text in the tabs is right-justified.

  The following functions provide a means of getting and setting the default
  style for this class:
    defaultStyle    - Returns the current default style.  This is the same as
                      classDefaultStyle unless setDefaultStyle has been
                      called.
    setDefaultStyle - Sets the default style for all subsequent notebooks.
-------------------------------------------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, INotebook, IWindow, IControl);
                                    // style class definition
static const Style
  classDefaultStyle,
  spiralBinding,
  solidBinding,
  backPagesBottomRight,
  backPagesBottomLeft,
  backPagesTopRight,
  backPagesTopLeft,
  majorTabsRight,
  majorTabsLeft,
  majorTabsTop,
  majorTabsBottom,
  squareTabs,
  roundedTabs,
  polygonTabs,
  statusTextLeft,
  statusTextRight,
  statusTextCenter,
  tabTextLeft,
  tabTextRight,
  tabTextCenter;

static Style
  defaultStyle    ( );

static void
  setDefaultStyle ( const Style& style );

/*------------------------ Constructors ----------------------------------------
| You can create an instance of this class in the following ways:              |
|    - From a control ID, parent and owner windows, rectangle, and style.      |
|      This creates the specified notebook control and an object for it.       |
|    - From the ID of a notebook control on a dialog window.  This creates     |
|      the object for the specified notebook control.                          |
|    - From the window handle of an existing notebook control.  This creates   |
|      the object for the specified notebook control.                          |
------------------------------------------------------------------------------*/
  INotebook   ( unsigned long        windowId,
                IWindow*             parent,
                IWindow*             owner,
                const IRectangle&    initial = IRectangle(),
                const Style&         style = defaultStyle() );
  INotebook   ( unsigned long        windowId,
                IWindow*             parentAndOwner );
  INotebook   ( const IWindowHandle& handle );

virtual
  INotebook :: ~INotebook ( );

/*------------------------- Enumerations ---------------------------------------
| The following enumerations are defined:                                      |
|   Orientation   - Enumeration defines the location of the back page's corner |
|                   and the major tabs.  The values are:                       |
|                     backpagesBottomTabsRight - Back pages on the bottom      |
|                                                right corner and the major    |
|                                                tabs on the right side of     |
|                                                the notebook.                 |
|                     backpagesTopTabsRight    - Back pages on the top right   |
|                                                corner and the major tabs on  |
|                                                the right side of the         |
|                                                notebook.                     |
|                     backpagesBottomTabsLeft  - Back pages on the bottom      |
|                                                left corner and the major     |
|                                                tabs on the left side of the  |
|                                                notebook.                     |
|                     backpagesTopTabsLeft     - Back pages on the top left    |
|                                                corner and the major tabs on  |
|                                                the left side of the          |
|                                                notebook.                     |
|                     backpagesRightTabsTop    - Back pages on the top right   |
|                                                corner and the major tabs on  |
|                                                the top side of the notebook. |
|                     backpagesLeftTabsTop     - Back pages on the top left    |
|                                                corner and the major tabs on  |
|                                                the top side of the notebook. |
|                     backpagesRightTabsBottom - Back pages on the bottom      |
|                                                right corner and the major    |
|                                                tabs on the bottom side of    |
|                                                the notebook.                 |
|                     backpagesLeftTabsBottom  - Back pages on the bottom      |
|                                                left corner and the major     |
|                                                tabs on the bottom side of    |
|                                                the notebook.                 |
|                                                                              |
|   TabShape      - Enumeration that defines the shape of the notebook tabs.   |
|                   The values are:                                            |
|                     square  - Draws the tabs with square corners.            |
|                     rounded - Draws the tabs with rounded corners.           |
|                     polygon - Draws the tabs with polygon corners.           |
|                                                                              |
|   TextAlignment - Enumeration that defines the alignment for the text in     |
|                   the status line and in the tabs.  The values are:          |
|                     left   - Left-justifies the text.                        |
|                     right  - Right-justifies the text.                       |
|                     center - Centers the text.                               |
|                                                                              |
|   Binding       - Enumeration that defines the binding for the notebook.     |
|                   The values are:                                            |
|                     spiral - Draws a spiral binding for the notebook.        |
|                     solid  - Draws a solid binding for the notebook.         |
------------------------------------------------------------------------------*/
enum Orientation {
  backpagesBottomTabsRight,
  backpagesTopTabsRight,
  backpagesBottomTabsLeft,
  backpagesTopTabsLeft,
  backpagesRightTabsTop,
  backpagesLeftTabsTop,
  backpagesRightTabsBottom,
  backpagesLeftTabsBottom
};

enum TabShape {
  square,
  rounded,
  polygon
};

enum TextAlignment {
  left,
  right,
  center
};

enum Binding {
  spiral,
  solid
};

/*--------------------------- Style Functions ----------------------------------
| The following functions provide a means of changing the notebook's styles:   |
|   setBinding             - Sets the type of binding with which the           |
|                            notebook will be drawn.                           |
|   setOrientation         - Sets the type of orientation of the notebook,     |
|                            which is the location of the major tabs and back  |
|                            pages.                                            |
|   setTabShape            - Sets the shape of the notebook's tabs.            |
|   setStatusTextAlignment - Sets the alignment of the text in the notebook's  |
|                            status line.                                      |
|   setTabTextAlignment    - Sets the alignment of the text in the notebook's  |
|                            tabs.                                             |
|                                                                              |
| The following functions provide a means of querying the notebook's styles:   |
|   binding             - Returns the type of binding used for the notebook.   |
|   orientation         - Returns the current orientation of the notebook,     |
|                         which is the location of the major tabs and the      |
|                         back pages.                                          |
|   tabShape            - Returns the shape of the notebook's tabs.            |
|   statusTextAlignment - Returns the alignment of the text in the notebook's  |
|                         status line.                                         |
|   tabTextAlignment    - Returns the alignment of the text in the notebook's  |
|                         tabs.                                                |
------------------------------------------------------------------------------*/
virtual INotebook
 &setBinding              ( Binding       binding ),
 &setOrientation          ( Orientation   orientation ),
 &setTabShape             ( TabShape      tabShape ),
 &setStatusTextAlignment  ( TextAlignment alignment ),
 &setTabTextAlignment     ( TextAlignment alignment );

virtual Binding
  binding                 ( ) const;
virtual Orientation
  orientation             ( ) const;
virtual TabShape
  tabShape                ( ) const;
virtual TextAlignment
  statusTextAlignment     ( ) const,
  tabTextAlignment        ( ) const;

/*-------------------------- Size Functions ------------------------------------
| The following functions provide a means of changing the size of notebook's   |
| parts:                                                                       |
|   setMajorTabSize   - Sets the size of the notebook's major tabs, in pixels. |
|   setMinorTabSize   - Sets the size of the notebook's minor tabs, in pixels. |
|   setPageButtonSize - Sets the size of the arrow buttons used to turn the    |
|                       notebook's pages, in pixels.                           |
------------------------------------------------------------------------------*/
virtual INotebook
 &setMajorTabSize   ( const ISize& sizeMajorTab ),
 &setMinorTabSize   ( const ISize& sizeMinorTab ),
 &setPageButtonSize ( const ISize& sizePageButton );

/*---------------------- Refresh Tab Function ----------------------------------
| The following function provides a means of operating on a notebook's tabs:   |
|   refreshTabs - Causes all of the notebook's tabs to be repainted.           |
------------------------------------------------------------------------------*/
virtual INotebook
 &refreshTabs       ( );

/*----------------------- Color Enumerations -----------------------------------
| The following color enumerations are defined:                                |
|   ColorArea - Enumeration that defines areas of the notebook that can be     |
|               colored.  The values are:                                      |
|                 pageBackground           - The page's background area.       |
|                 majorTabBackground       - Background of the major tabs.     |
|                 majorTabForeground       - Foreground of the major tabs.     |
|                 minorTabBackground       - Background of the minor tabs.     |
|                 minorTabForeground       - Foreground of the minor tabs.     |
|                 notebookWindowBackground - Background of the notebook's      |
|                                            window.                           |
|                 notebookOutline          - Outline of the notebook.          |
|                 statusLineText           - Text on the notebook's status     |
|                                            line.                             |
|                 selectionCursor          - The notebook's selection cursor.  |
------------------------------------------------------------------------------*/
enum ColorArea {
  pageBackground,
  majorTabBackground,
  majorTabForeground,
  minorTabBackground,
  minorTabForeground,
  notebookWindowBackground,
  notebookOutline,
  statusLineText,
  selectionCursor
};

/*------------------------- Color Functions ------------------------------------
| The following functions provide a means for setting and querying the color of|
| different parts of the notebook:                                             |
|   setColor - Sets the color of a part of the notebook.                       |
|   color    - Queries the color of a part of the notebook.                    |
------------------------------------------------------------------------------*/
virtual INotebook
 &setColor        ( ColorArea      area,
                    const IColor&  color );
virtual IColor
  color           ( ColorArea      area ) const;


class PageSettings : public IVBase {
typedef IVBase
  Inherited;
/*******************************************************************************
* The INotebook::PageSettings nested class is used to identify information     *
* about a page in the notebook.  PageSettings objects are typically created    *
* as a means of defining pages' characteristics when adding them to a          *
* notebook.  A PageSettings object can also be created to reflect the current  *
* state of a page of the notebook.  The PageSettings only identifies the       *
* characteristics of a notebook page at one point in time.  It is not updated  *
* when the actual notebook page changes.  Also, changes to the PageSettings    *
* are not reflected in the actual notebook page after the page is added to     *
* the notebook.                                                                *
*******************************************************************************/
friend class INotebook;
public:
/*--------------- Attribute -----------------------------------------------
  The INotebook::PageSettings::Attribute nested class provides static members that
  define the set of valid notebook page attributes.  For example, you could
  define an instance of the INotebook::Page::Attribute class and initialize
  it like:
              INotebook::PageSettings::Attribute
                attribute = INotebook::PageSettings::majorTab;
  An object of this type is provided when the notebook page is created.

  The declaration of the INotebook::PageSettings::Attribute nested class is generated
  by the INESTEDBITFLAGCLASSDEF0 macro.

  The valid notebook page attributes are:
    autoPageSize - The notebook handles the positioning and sizing of the
                   page.
    statusTextOn - Specifies that the page will have a status line.
    majorTab     - Specifies that the page will be associated with a major
                   tab.
    minorTab     - Specifies that the page will be associated with a minor
                   tab.
    noAttribute  - Specifies that no attribute applies to the notebook page.
-------------------------------------------------------------------------*/
INESTEDBITFLAGCLASSDEF0(Attribute, INotebook::PageSettings);
static const Attribute
  autoPageSize,
  statusTextOn,
  majorTab,
  minorTab,
  noAttribute;

/*------------------- Constructors ---------------------------------------------
| You can construct instances of this class in the following ways:             |
|    - Using the default.  Attribute is defined to be noAttribute, and all     |
|      the other page settings values are set to NULL or 0.                    |
|    - Using one or more page attributes.  All the other page settings are     |
|      set to NULL or 0.                                                       |
|    - Using the tab text, status text, and one or more attributes.  In this   |
|      case, it is assumed that there is no bit map for this tab.              |
|    - Using the tab bit map, status text, and one or more attributes.  In     |
|      this case, it is assumed that there is no text for this tab.            |
------------------------------------------------------------------------------*/
  PageSettings  ( );
  PageSettings  ( Attribute            attribute );
  PageSettings  ( const char*          tabText,
                  const char*          statusText = 0,
                  Attribute            pageAttribute = noAttribute );
  PageSettings  ( const IBitmapHandle& tabBitmap,
                  const char*          statusText = 0,
                  Attribute            pageAttribute = noAttribute );
virtual
  PageSettings :: ~ PageSettings ( );

  PageSettings :: PageSettings (const INotebook::PageSettings&);

/*------------------- Page Settings Functions ----------------------------------
| The following functions set and query the data associated with a PageSettings|
| object:                                                                      |
|   setStatusText - Sets the text of the status line associated with this      |
|                   page.                                                      |
|   setTabText    - Sets the text of the tab associated with this page.        |
|   setTabBitmap  - Sets the bit map of the tab associated with this page.     |
|   setUserData   - Sets the application data in the page's reserved storage.  |
|   statusText    - Returns the text of the status line associated with this   |
|                   page.                                                      |
|   tabText       - Returns the text of the tab associated with this page.     |
|   tabBitmap     - Returns the bit map of the tab associated with this page.  |
|   userData      - Returns the application data in the page's reserved        |
|                   storage.                                                   |
| Once a PageSettings object has been used to insert a page in the notebook,   |
| using these functions on the PageSettings object will have no effect on the  |
| actual notebook; the corresponding notebook functions should be used         |
| instead.  However, these functions could be used to update the PageSettings  |
| object for defining a new notebook page.                                     |
------------------------------------------------------------------------------*/
virtual INotebook::PageSettings
 &setStatusText     ( const char*          statusText ),
 &setStatusText     ( const IResourceId&   resourceId ),
 &setTabText        ( const char*          tabText ),
 &setTabText        ( const IResourceId&   resourceId ),
 &setTabBitmap      ( const IBitmapHandle& bitmap ),
 &setTabBitmap      ( const IResourceId&   resourceId ),
 &setUserData       ( unsigned long        userData );

virtual IString
  statusText        ( ) const,
  tabText           ( ) const;

virtual IBitmapHandle
  tabBitmap         ( ) const;

virtual unsigned long
  userData          ( ) const;

/*-------------------------- Attribute Functions -------------------------------
| The following functions test a page for the presence of certain attributes:  |
|   isAutoSize     - Queries whether the page's size is handled by the         |
|                    notebook.                                                 |
|   isStatusTextOn - Queries whether a status line is associated with the      |
|                    page.                                                     |
|   isMajorTab     - Queries whether a major tab is associated with the page.  |
|   isMinorTab     - Queries whether a minor tab is associated with the page.  |
------------------------------------------------------------------------------*/
Boolean
  isAutoSize       ( ) const,
  isStatusTextOn   ( ) const,
  isMajorTab       ( ) const,
  isMinorTab       ( ) const;

private:
/*--------------------------------- Private ----------------------------------*/
Attribute
  pageStyle;
IBitmapHandle
  savedTabBitmap;
unsigned long
  savedUserData;
IString
  savedStatusText,
  savedTabText;

}; // INotebook::PageSettings


class Cursor : public IVBase {
typedef IVBase
  Inherited;
/*******************************************************************************
* The INotebook::Cursor nested class defines objects that can be used to       *
* traverse the pages of a notebook.  In the same way that a Cursor can be      *
* used to traverse the objects in a Collection, this Cursor can be used to go  *
* through a notebook, one page at a time.  The notebook's Cursor deals with    *
* IPageHandles, each of which can be thought of as a pointer to a specific     *
* page in the notebook.                                                        *
*******************************************************************************/
public :
/*------------------- Constructors ---------------------------------------------
| You can construct instances of this class with the notebook whose pages      |
| this Cursor instance is going to traverse.                                   |
------------------------------------------------------------------------------*/
  Cursor  ( const INotebook& notebook );

virtual
  ~Cursor ( );

/*------------------------ Cursor Movement -------------------------------------
| The following functions provide a means of moving the cursor position        |
| within the notebook with which it is associated:                             |
|   setToFirst    - Sets the cursor to point to the first page of the          |
|                   notebook.                                                  |
|   setToNext     - Sets the cursor to point to the next page of the notebook. |
|   setToPrevious - Sets the cursor to point to the previous page of the       |
|                   notebook.                                                  |
|   setToLast     - Sets the cursor to point to the last page of the notebook. |
------------------------------------------------------------------------------*/
virtual Boolean
  setToFirst    ( ),
  setToNext     ( ),
  setToPrevious ( ),
  setToLast     ( );

/*------------------------ Page Retrieval --------------------------------------
| The following functions are used to retrieve pages:                          |
|   first      - Points to the first page in the notebook and returns an       |
|                IPageHandle object that points to the first notebook page.    |
|   next       - Points to the next page in the notebook and returns an        |
|                IPageHandle object that points to the next notebook page.     |
|   previous   - Points to the previous page in the notebook and returns an    |
|                IPageHandle object that points to the previous notebook page. |
|   last       - Points to the last page in the notebook and returns an        |
|                IPageHandle object that points to the last notebook page.     |
|   current    - Returns an IPageHandle object for the notebook page to which  |
|                the cursor currently points.                                  |
|   setCurrent - Uses a specified IPageHandle to set the cursor to a           |
|                particular notebook page.                                     |
------------------------------------------------------------------------------*/
virtual IPageHandle
  first      ( ),
  next       ( ),
  previous   ( ),
  last       ( ),
  current    ( ) const;
void
  setCurrent ( const IPageHandle& current );

/*--------------- Cursor Validation --------------------------------------------
| The following functions are used to validate the cursor:                     |
|   isValid    - Queries whether the cursor points to a valid item.            |
|   invalidate - Flags this cursor as invalid.                                 |
-------------------------------------------------------------------------------*/
virtual Boolean
  isValid    ( ) const;
virtual void
  invalidate ( );

private :
/*--------------------------------- Private ----------------------------------*/
const INotebook
 &nbCl;
unsigned long
  ulClCurrentPageId;
unsigned long
  ulClReferences;
}; // INotebook::Cursor


/*-------------------------------- Page Operations -----------------------------
| The following functions handle the manipulation of pages within a notebook:  |
| NOTE: When adding an IFrameWindow as a notebook page, you should create an   |
|       IFrameWindow with its parent as the notebook.  This applies to the     |
|       addFirstPage, addLastPage, addPageAfter, and addPageBefore functions.  |
|   addFirstPage     - Adds a window as the first page in the notebook using   |
|                      settings specified in a PageSettings object.  If the    |
|                      window pointer is NULL, a page with the indicated       |
|                      settings, but no window, is added to the notebook.      |
|   addLastPage      - Adds a window as the last page in the notebook using    |
|                      settings specified in a PageSettings object.  If the    |
|                      window pointer is NULL, a page with the indicated       |
|                      settings, but no window, is added to the notebook.      |
|   addPageAfter     - Adds a window to the notebook following the page        |
|                      referenced by a Cursor or an IPageHandle using          |
|                      settings specified in a PageSettings object.  If the    |
|                      window pointer is NULL, a page with the indicated       |
|                      settings, but no window, is added to the notebook.      |
|   addPageBefore    - Adds a window to the notebook before the page           |
|                      referenced by a Cursor or an IPageHandle using          |
|                      settings specified in a PageSettings object.  If the    |
|                      window pointer is NULL, a page with the indicated       |
|                      settings, but no window, is added to the notebook.      |
|   removePage       - Removes a specified page from the notebook.             |
|   removeAllPages   - Removes all the pages in the notebook.                  |
|   removeTabSection - If the page specified is a major tab page, removes the  |
|                      specified page and all subsequent pages up to the next  |
|                      major tab page.  If the page specified is a minor tab   |
|                      page, removes the specified page and all subsequent     |
|                      pages up to the next page that has a tab.               |
|   turnToPage       - Moves a specified page to the top of the notebook.      |
|   topPage          - Returns an IPageHandle object that references the       |
|                      current top page of the notebook.                       |
|   firstPage        - Returns an IPageHandle object that references the       |
|                      first page in the notebook.                             |
|   lastPage         - Returns an IPageHandle object that references the last  |
|                      page in the notebook.                                   |
|   nextPage         - Returns an IPageHandle object that references the page  |
|                      following the specified IPageHandle.                    |
|   previousPage     - Returns an IPageHandle object that references the page  |
|                      before the specified IPageHandle.                       |
|   window           - Returns the window associated with the specified        |
|                      notebook page.                                          |
|   pageSettings     - Returns an instance of the page settings.               |
|   setStatusText    - Sets the status text for a specified page of the        |
|                      notebook.                                               |
|   setTabText       - Sets the tab text for a specified page of the notebook. |
|   setTabBitmap     - Sets the tab bit map for a specified page of the        |
|                      notebook.                                               |
|   setUserData      - Sets the user data for a specified page of the          |
|                      notebook.                                               |
|   setWindow        - Associates the specified page with the window.          |
------------------------------------------------------------------------------*/
virtual IPageHandle
  addFirstPage      ( const PageSettings& pageInfo,
                      IWindow*            window = 0 ),
  addLastPage       ( const PageSettings& pageInfo,
                      IWindow*            window = 0 ),
  addPageBefore     ( const PageSettings& pageInfoToAdd,
                      const IPageHandle&  referencePage,
                      IWindow*            window = 0 ),
  addPageAfter      ( const PageSettings& pageToAdd,
                      const IPageHandle&  referencePage,
                      IWindow*            window = 0 ),
  addPageBefore     ( const PageSettings& pageInfo,
                      const Cursor&       cursor,
                      IWindow*            window = 0 ),
  addPageAfter      ( const PageSettings& pageInfo,
                      const Cursor&       cursor,
                      IWindow*            window = 0 );
virtual INotebook
 &removePage        ( const IPageHandle&  page ),
 &removePage        ( const Cursor&       cursor ),
 &removeAllPages    ( ),
 &removeTabSection  ( const IPageHandle&  page ),
 &removeTabSection  ( const Cursor&       cursor ),
 &turnToPage        ( const IPageHandle&  page ),
 &turnToPage        ( const Cursor&       cursor );

virtual IPageHandle
  topPage           ( ) const,
  firstPage         ( ) const,
  lastPage          ( ) const,
  nextPage          ( const IPageHandle&  referencePage ) const,
  previousPage      ( const IPageHandle&  referencePage ) const;

virtual IWindow
 *window            ( const Cursor&       cursor ) const,
 *window            ( const IPageHandle&  page ) const;

PageSettings
  pageSettings      ( const Cursor&       cursor ) const,
  pageSettings      ( const IPageHandle&  page ) const;

virtual INotebook
 &setStatusText     ( const IPageHandle&   referencePage,
                      const char*          statusText ),
 &setStatusText     ( const IPageHandle&   referencePage,
                      const IResourceId&   resourceId ),
 &setTabText        ( const IPageHandle&   referencePage,
                      const char*          tabText ),
 &setTabText        ( const IPageHandle&   referencePage,
                      const IResourceId&   resourceId ),
 &setTabBitmap      ( const IPageHandle&   referencePage,
                      const IBitmapHandle& bitmap ),
 &setTabBitmap      ( const IPageHandle&   referencePage,
                      const IResourceId&   resourceId ),
 &setUserData       ( const IPageHandle&   referencePage,
                      unsigned long        userData ),
 &setWindow         ( const Cursor&        cursor,
                            IWindow*       window = 0 ),
 &setWindow         ( const IPageHandle&   referencePage,
                            IWindow*       window = 0 );


/*-------------------------- Number Functions ----------------------------------
| The following functions provide number and size information for the          |
| notebook:                                                                    |
|   pageSize        - Returns the page's size.  All notebook pages have the    |
|                     same size.                                               |
|   notebookSize    - Returns the size that the notebook needs to be in order  |
|                     to contain the page.                                     |
|   totalPages      - Returns the number of pages in the notebook.             |
|   pagesToMinorTab - Returns the number of pages in the notebook between a    |
|                     specified page and the next page that has a minor tab.   |
|   pagesToMajorTab - Returns the number of pages in the notebook between a    |
|                     specified page and the next page that has a major tab.   |
|   pagesToEnd      - Returns the number of pages in the notebook from a       |
|                     specified page to the end of the notebook.  This value   |
|                     includes the specified page.                             |
|   isEmpty         - Queries whether the notebook has any pages.              |
------------------------------------------------------------------------------*/
virtual ISize
  pageSize        ( ) const,
  notebookSize    ( const IPageHandle& page ) const;

virtual unsigned long
  totalPages      ( ) const,
  pagesToMajorTab ( const IPageHandle& page ) const,
  pagesToMajorTab ( const Cursor&      cursor ) const,
  pagesToMinorTab ( const IPageHandle& page ) const,
  pagesToMinorTab ( const Cursor&      cursor ) const,
  pagesToEnd      ( const IPageHandle& page ) const,
  pagesToEnd      ( const Cursor&      cursor ) const;

virtual Boolean
  isEmpty         ( ) const;

protected:
/*----------------------------- Layout Size ------------------------------------
| calcMinimumSize - Returns the minimum recommended size for this notebook.    |
|                   This size is large enough to contain the largest page      |
|                   window that is currently in the notebook.                  |
------------------------------------------------------------------------------*/
virtual ISize
  calcMinimumSize ( ) const;

private:
/*--------------------------------- Private ----------------------------------*/
static Style
  currentDefaultStyle;

unsigned long
  queryPage      ( void* mp1, void* mp2 ) const,
  insertPage     ( void* mp1, void* mp2 );
Boolean
  deletePage     ( void* mp1, void* mp2 );
short
  queryPageCount ( void* mp1, void* mp2 ) const;
IPageHandle
  insertPageInfo ( const PageSettings& pageInfo,
                   const IPageHandle&  referencePage,
                   IWindow*            window,
                   unsigned long       fPosition );
ITabBitmapMgr
  *bmClTabBitmapMgr;
unsigned long
  ulClValidate;

friend class INotebook::Cursor;
}; // INotebook

INESTEDBITFLAGCLASSFUNCS(Style, INotebook);

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing and warning messages.                      */
/*----------------------------------------------------------------------------*/
#pragma pack()
#pragma info(restore)

/*--------------------------------- Inlines ----------------------------------*/
#ifndef I_NO_INLINES
  #include <inotebk.inl>
#endif

#endif /* _INOTEBK_ */
