#ifndef _ISCROLL_
  #define _ISCROLL_
/*******************************************************************************
* FILE NAME: iscroll.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*    IScrollBar - This class creates and manages the scroll bar control.       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif
#ifndef _IRECT_
  #include <irect.hpp>
#endif

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:
class IRange;
class IWindowHandle;
struct _SBCDATA;
struct _WNDPARAMS;


class IScrollBar : public IControl  {
typedef IControl
  Inherited;
/*******************************************************************************
* The IScrollBar class creates and manages the scroll bar control window.      *
*                                                                              *
* Example:                                                                     *
* To create a horizontal scroll bar:                                           *
*    IScrollBar* pvsb = new IScrollBar(ID_MYHORZSCROLL,                        *
*                                      this, this,                             *
*                                      IRange(1, 20),                          *
*                                      5, 1,                                   *
*                                      IRectangle(50, 20, 200, 40),            *
*                                      IWindow::visible |                      *
*                                        IScrollBar::horizontal);              *
*                                                                              *
* Then to catch the scroll messages, add:                                      *
*    MyScrollHandler* psbh = new MyScrollHandler();                            *
*    psbh->handleEventsFor(this);                                              *
*                                                                              *
* Finally, to process the scroll messages:                                     *
*    Boolean  MyScrollHandler::pageUp(IScrollEvent& evt)                       *
*    {                                                                         *
*       // Scroll the information in the related window                        *
*       ...                                                                    *
*       moveScrollBox(evt);  // default processing to position scroll box      *
*       return true;                                                           *
*    }                                                                         *
*******************************************************************************/
public:
class Style;                  // forward declaration for nested class

/*------------------------- Constructors/Destructor ----------------------------
| You can construct instances of this class in the following ways:             |
|    - From a control ID, parent and owner windows, rectangle, and style.      |
|      This creates the specified scroll bar control and an object for it.     |
|      The range of items is defaulted to (1, 100), the number of visible      |
|      items to 25, and the scroll box position to 1.                          |
|    - From a control ID, parent and owner windows, rectangle, range of all    |
|      scrollable items, number of items that are displayed, scroll box        |
|      position, and style.  This creates the specified scroll bar control     |
|      and an object for it.                                                   |
|    - From the ID of a scroll bar control on a dialog window.  This creates   |
|      the object for the specified scroll bar control.                        |
|    - From the window handle of an existing scroll bar control.  This         |
|      creates the object for the specified scroll bar control.                |
------------------------------------------------------------------------------*/
  IScrollBar ( unsigned long windowId,
               IWindow* parent,
               IWindow* owner,
               const IRectangle& initial = IRectangle(),
               const Style& style = defaultStyle() );

  IScrollBar ( unsigned long windowId,
               IWindow* parent,
               IWindow* owner,
               const IRange& scrollableItems,
               unsigned long visibleItemCount,
               unsigned long scrollBoxPosition = 1,
               const IRectangle& initial = IRectangle(),
               const Style& style = defaultStyle() );

  IScrollBar ( unsigned long windowId, IWindow* parent );
  IScrollBar ( const IWindowHandle& handle );

virtual
 ~IScrollBar ( );

/*---------------------------------- Style -------------------------------------
  The following functions provide a means to set and query scroll bar styles:

    Style - Nested class that provides static members that define the set of
            valid scroll bar styles.  These styles can be used in
            conjunction with the styles defined by the nested classes
            IWindow::Style and IControl::Style.  For example, you could
            define an instance of the IScrollBar::Style class and initialize
            it like:
              IScrollBar::Style
                style = IScrollBar::horizontal | IWindow::visible;
            An object of this type is provided when the scroll bar is
            created.  A customizable default is used if no styles are
            specified.  Once the object is constructed, IScrollBar, IWindow,
            and IControl member functions can be used to set or query the
            object's style.

            The declaration of the IScrollBar::Style nested class is
            generated by the INESTEDBITFLAGCLASSDEF2 macro.

  The valid scroll bar styles are:
    horizontal        - Causes a horizontal scroll bar to be created.  This
                        style is ignored if it is used with vertical.
    vertical          - Causes a vertical scroll bar to be created.  This
                        style is used if both it and horizontal are specified.
    autoSize          - Causes automatic sizing of the scroll bar to occur.
                        For a vertical scroll bar, the width is determined.
                        For a horizontal scroll bar, the height is determined.
    classDefaultStyle - Original default style for this class, which is
                        IWindow::visible | vertical.
------------------------------------------------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, IScrollBar, IWindow, IControl);
static const Style
  horizontal,
  vertical,
  autoSize,
  classDefaultStyle;

/*------------------------------ Default Style ---------------------------------
| These functions provide a means of getting and setting the default style     |
| attributes of instances of this class:                                       |
|   defaultStyle    - Returns the current default style.  This is the same as  |
|                     classDefaultStyle unless setDefaultStyle has been        |
|                     called.                                                  |
|   setDefaultStyle - Sets the default style for all subsequent scroll bars.   |
------------------------------------------------------------------------------*/
static Style
  defaultStyle        ( );
static void
  setDefaultStyle     ( const Style& style );

/*------------------------------ System Values ---------------------------------
| The following static functions return various system sizes.  For the below   |
| functions, length refers to the size in the direction of a scroll bar's      |
| orientation (that is, in the X-direction for a horizontal scroll bar, in the |
| Y-direction for a vertical scroll bar).  Width refers to the size in the     |
| direction perpendicular to the scroll bar's orientation (that is, in the     |
| Y-direction for a horizontal scroll bar, in the X-direction for a vertical   |
| scroll bar).                                                                 |
|   systemScrollBarWidth     - Returns the system width of a vertical or       |
|                              horizontal scroll bar.                          |
|   systemScrollBoxLength    - Returns the system length of a scroll box for   |
|                              a vertical or horizontal scroll bar.            |
|   systemScrollButtonLength - Returns the system length of a scroll button    |
|                              for a vertical or horizontal scroll bar.        |
------------------------------------------------------------------------------*/
static unsigned long
  systemScrollBarWidth     ( Boolean verticalScrollBar = true ),
  systemScrollBoxLength    ( Boolean verticalScrollBar = true ),
  systemScrollButtonLength ( Boolean verticalScrollBar = true );

/*-------------------- Query, Enable, and Disable Styles -----------------------
| These functions provide a means of getting and setting the scroll bar        |
| styles:                                                                      |
|   isHorizontal - Returns true if the scroll bar is horizontal.               |
|   isVertical   - Returns true if the scroll bar is vertical.                 |
------------------------------------------------------------------------------*/
Boolean
  isHorizontal        ( ) const,
  isVertical          ( ) const;

/*------------------------- Manipulation Operations ----------------------------
| These functions change and query how the scroll bar operates:                |
|   setScrollableRange - Sets the range of all items to which the scroll bar   |
|                        can scroll.                                           |
|   setVisibleCount    - Defines the amount of the scrollable range that is    |
|                        displayed.  This is used to determine the length of   |
|                        the scroll box, and the default value of the page     |
|                        scrolling increment.                                  |
|   setScrollBar       - Sets the range of all items and the number of items   |
|                        that are displayed.  This is functionally equivalent  |
|                        to calling both setScrollableRange and                |
|                        setVisibleCount.                                      |
|   moveScrollBoxTo    - Moves the scroll box to the position given.           |
|   scrollableRange    - Returns the range that the scroll bar represents.     |
|   scrollBoxRange     - Returns the range in which the scroll box can be      |
|                        positioned.  This is defined to be:                   |
|                          IRange(scrollableRange().lowerBound(),              |
|                                 scrollableRange().upperBound() -             |
|                                   visibleCount())                            |
|   visibleCount       - Returns the amount of the scrollable range that is    |
|                        displayed.  This is used to determine the length of   |
|                        the scroll box, and the default value of the page     |
|                        scrolling increment.                                  |
|   scrollBoxPosition  - Returns the current position of the scroll box.       |
------------------------------------------------------------------------------*/
IScrollBar
 &setScrollableRange  ( const IRange& minMax ),
 &setVisibleCount     ( unsigned long scrollableRangeUnits ),
 &setScrollBar        ( const IRange& scrollableRange,
                        unsigned long visibleCount ),
 &moveScrollBoxTo     ( unsigned long firstItem = 1 );
IRange
  scrollableRange     ( ) const,
  scrollBoxRange      ( ) const;
unsigned long
  visibleCount        ( ) const,
  scrollBoxPosition   ( ) const;

/*--------------------------- Scrolling Increment ------------------------------
| These functions query and set the scrolling increments for the scroll bar.   |
| These values are used to determine the scroll amounts stored in              |
| IScrollEvent objects.                                                        |
|   minScrollIncrement     - Returns the amount of the scrollable range that   |
|                            is scrolled by selecting the scroll buttons.      |
|   pageScrollIncrement    - Returns the amount of the scrollable range that   |
|                            is scrolled by selecting the scroll shaft.        |
|   setMinScrollIncrement  - Sets the amount of the scrollable range that is   |
|                            scrolled by selecting the scroll buttons.         |
|   setPageScrollIncrement - Sets the amount of the scrollable range that is   |
|                            scrolled by selecting the scroll shaft.           |
|                            Specifying a value of 0 means to use the visible  |
|                            count minus the minimum scrolling increment.      |
|                            This is the default value.  (If the visible       |
|                            count minus the minimum scrolling increment is    |
|                            equal to 0, the minimum scrolling increment is    |
|                            used).                                            |
------------------------------------------------------------------------------*/
unsigned long
  minScrollIncrement     ( ) const,
  pageScrollIncrement    ( ) const;
IScrollBar
 &setMinScrollIncrement  ( unsigned long scrollableRangeUnits = 1 ),
 &setPageScrollIncrement ( unsigned long scrollableRangeUnits = 0 );

/*-------------------------- Color Area Enumerators ----------------------------
| ColorArea  - Identifies parts of a scroll bar whose color can be changed.    |
|              Values are:                                                     |
|              shaft     - The portion of the scroll bar not occupied by the   |
|                          scroll box or the scroll buttons.                   |
|              scrollBox - The portion of a scroll bar that represents the     |
|                          visible part of the information being scrolled.     |
|              border    - The color of the border that surrounds the scroll   |
|                          bar window.                                         |
------------------------------------------------------------------------------*/
enum ColorArea {
  shaft,
  scrollBox,
  border
};

/*---------------------------------- Color -------------------------------------
|  setColor - Changes the color of the specified portion of the scroll bar.    |
|  color    - Returns the color of the specified portion of the scroll bar.    |
------------------------------------------------------------------------------*/
IScrollBar
 &setColor              ( ColorArea area, const IColor& color );
IColor
  color                 ( ColorArea value ) const;

protected:
/*--------------------------- Protected Overrides ------------------------------
| This class overrides the following inherited protected functions:            |
|   calcMinimumSize - Returns the recommended minimum size of this scroll bar  |
|                     control.                                                 |
------------------------------------------------------------------------------*/
virtual ISize
  calcMinimumSize       ( ) const;

private:
/*--------------------------------- Private ----------------------------------*/
  IScrollBar            ( const IScrollBar& );
IScrollBar
 &operator=             ( const IScrollBar& );
void
  initialize            ( unsigned long windowId,
                          IWindow* parent,
                          IWindow* owner,
                          const IRectangle& initial,
                          const IRange& scrollableItems,
                          unsigned long visibleItemCount,
                          unsigned long scrollBoxPosition,
                          const Style& style ),
  initializeControlData ( const IRange& scrollableItems,
                          unsigned long visibleItemCount,
                          struct _SBCDATA* controlData ),
  windowData            ( struct _WNDPARAMS* windowData ) const;

static Style
  currentDefaultStyle;
unsigned long
  ulClMinScrollIncrement,
  ulClPageScrollIncrement;
}; // IScrollBar

INESTEDBITFLAGCLASSFUNCS(Style, IScrollBar);

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*----------------------------- Inline Functions -----------------------------*/
#ifndef I_NO_INLINES
  #include <iscroll.inl>
#endif
#endif /* _ISCROLL_ */
